-----------------------------------------------------------------------------

local AssessmentCombatTime = Assessment:NewModule("CombatTime", "AceEvent-2.0")
local Roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------------

local new, reset = Assessment.NewTable, Assessment.ResetTable
local pairs = pairs
local UnitAffectingCombat, GetTime = UnitAffectingCombat, GetTime

-----------------------------------------------------------------------------

Assessment.TagData.CurrentCombatDuration =
{
	method = function(data) 
				local fightData = data.fightData
				if not fightData then return 0 end
				return fightData.combatTime.currentDuration or 0
			 end,
	data = "combatTime",
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.CombatDuration =
{
	method = function(data) 
				local fightData = data.fightData
				if not fightData then return 0 end
				return fightData.combatTime.duration or 0
			 end,
	data = "combatTime",
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.TotalDuration =
{
	method = function(data) 
				local fightData = data.fightData
				if not fightData then return 0 end
				return fightData.combatTime.totalDuration or 0
			 end,
	data = "combatTime",
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.IsInCombat =
{
	method = function(data, prior) 
				if not prior then return false end

				local fightData = data.fightData
				if not fightData then return false end

				return fightData.combatTime.durationOffset
			 end,
	data = "combatTime",
}
-----------------------------------------------------------------------------

function AssessmentCombatTime:OnInitialize()
	for data in self:ForceIterateUpdateDataTables() do
		if not data.combatTime then
			data.combatTime = new()
		end
	end
end

function AssessmentCombatTime:OnEnable()
	self:RegisterEvent("Assessment:ResetFightData", "OnResetFightData")
	self:RegisterEvent("Assessment:NewFightDataTable", "OnNewFightDataTable")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "OnCombatStart")
	self:RegisterEvent("RosterLib_RosterUpdated")

	self:RosterLib_RosterUpdated()	
end

function AssessmentCombatTime:OnDisable()
end

-----------------------------------------------------------------------------

local groupMemberUnitIdList = { }

function AssessmentCombatTime:RosterLib_RosterUpdated()
	if GetNumPartyMembers() + GetNumRaidMembers() > 0 then
		reset(groupMemberUnitIdList)

		for unit in Roster:IterateRoster() do
			groupMemberUnitIdList[unit.unitid] = true
		end
	
		self:ScheduleRepeatingEvent("Assessment:GroupCombatTimeUpdate", self.UpdateGroup, 0.2, self)
		self:CancelEvent("PLAYER_REGEN_ENABLED")
	else
		self:RegisterEvent("PLAYER_REGEN_ENABLED", "OnCombatEnd")
		self:CancelScheduledEvent("Assessment:GroupCombatTimeUpdate")
	end
end

-----------------------------------------------------------------------------

function AssessmentCombatTime:UpdateGroup()
	for k in pairs(groupMemberUnitIdList) do
		if UnitAffectingCombat(k) then
			self:OnCombatStart()
			return
		end
	end
	self:OnCombatEnd()
end

-----------------------------------------------------------------------------

local inCombat
function AssessmentCombatTime:OnResetFightData()
	if inCombat then
		self:DoCombatStart()
	end
end

function AssessmentCombatTime:IsInCombat()
	return inCombat
end

function AssessmentCombatTime:OnCombatStart()
	if inCombat then return end
	inCombat = true
	
	self:DoCombatStart()
end

function AssessmentCombatTime:OnNewFightDataTable(data)
	data.combatTime = new()
end

function AssessmentCombatTime:DoCombatStart()
	Assessment:StartNewFight()
	self:ScheduleRepeatingEvent("Assessment:CombatTimeUpdate", self.Update, 0, self)
	
	local time = GetTime()
	
	for data in self:IterateUpdateDataTables() do
		local combatTime = data.combatTime
		if combatTime.duration then
			combatTime.durationOffset	= combatTime.duration - time
			combatTime.totalDuration	= combatTime.totalDurationOffset + time
		else
			combatTime.durationOffset		= -time
			combatTime.totalDurationOffset	= -time
			combatTime.duration				= 0
			combatTime.totalDuration		= 0
		end
		combatTime.currentDurationOffset	= -time
		combatTime.currentDuration			= 0
	end
end

function AssessmentCombatTime:OnCombatEnd()
	if not inCombat then return end
	
	local time = GetTime()

	self:CancelScheduledEvent("Assessment:CombatTimeUpdate")
	for data in self:IterateUpdateDataTables() do
		local combatTime = data.combatTime

		combatTime.currentDuration	= combatTime.currentDurationOffset + time
		combatTime.duration			= combatTime.durationOffset + time
		combatTime.totalDuration	= combatTime.totalDurationOffset + time
		combatTime.durationOffset	= nil
	end

	inCombat = nil
end

function AssessmentCombatTime:Update()
	local time = GetTime()

	for data in self:IterateUpdateDataTables() do
		data.dataUpdateTimes.combatTime = time
		
		local combatTime = data.combatTime
		combatTime.currentDuration	= combatTime.currentDurationOffset + time
		combatTime.duration			= combatTime.durationOffset + time
		combatTime.totalDuration	= combatTime.totalDurationOffset + time
	end
end

-----------------------------------------------------------------------------
